// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.casetool;

import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.util.Debug;


public class UMLInfo {
    private final ListOfAssociation allAssociations;
    
    
    public UMLInfo(ListOfAssociation allAssociations) {
        this.allAssociations = allAssociations;
    }
    
    
    /**
     * Returns all associations with at least one end in the passed type.
     */
    public ListOfAssociation getAssociations(KeYJavaType kjt) {
        assert kjt != null;
        ListOfAssociation result = SLListOfAssociation.EMPTY_LIST;
        
        IteratorOfAssociation it = allAssociations.iterator();
        while(it.hasNext()) {
            Association assoc = it.next();
            
            IteratorOfAssociationEnd it2 = assoc.getEnds().iterator();
            while(it2.hasNext()) {
                AssociationEnd end = it2.next();
                
                if(kjt.getFullName().equals(end.getModelClass()
                                               .getFullClassName())) {
                    result = result.prepend(assoc);
                    break;
                }
            }
        }
        
        return result;
    }
    
    
    /**
     * Returns all binary associations with at least one end in the passed type,
     * and whose role name on the other side is the passed string.
     */
    public ListOfAssociation getAssociations(KeYJavaType kjt, 
                                             String qualifier) {
        ListOfAssociation result = SLListOfAssociation.EMPTY_LIST;
        
        //iterate over all associations for the desired class
        ListOfAssociation classAssocs = getAssociations(kjt);
        IteratorOfAssociation it = classAssocs.iterator();
        while(it.hasNext()) {
            Association assoc = it.next();
            
            ListOfAssociationEnd ends = assoc.getEnds();
            if(ends.size() != 2) {
                continue;
            }

            //identify possible "other side" ends
            ListOfAssociationEnd targetEnds = SLListOfAssociationEnd.EMPTY_LIST;
            String typeName = kjt.getFullName();
            if(ends.head().getModelClass().getFullClassName()
                                          .equals(typeName)) {
                targetEnds = targetEnds.prepend(ends.tail().head());
            }
            if(ends.tail().head().getModelClass().getFullClassName()
                                                 .equals(typeName)) {
                targetEnds = targetEnds.prepend(ends.head());
            }
            Debug.assertFalse(targetEnds.isEmpty());
            
            //check if one of those ends has the desired role name
            IteratorOfAssociationEnd it2 = targetEnds.iterator();
            while(it2.hasNext()) {
                AssociationEnd end = it2.next();                
                if(end.getRoleName().toString().equals(qualifier)) {
                    result = result.prepend(assoc);
                    break;
                }
            }
        }
        
        return result;
    }
}
