// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.gui;

import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Hashtable;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.strategy.*;

public class StrategySelectionView extends JPanel {

    final JRadioButtonHashMap bSimpleFOLStrategy = new JRadioButtonHashMap(
        "FOL", "FOLStrategy", false, false);
    final JRadioButtonHashMap bSimpleJavaCardDLStrategy = new JRadioButtonHashMap(
        "Java DL", "JavaCardDLStrategy", false, false);
    final JRadioButtonHashMap bSimplificationOfOCLStrategy = new JRadioButtonHashMap(
        "OCL simplification", "SimplificationOfOCLStrategy", false, false);
    final JRadioButtonHashMap bComputeSpecificationStrategy = new JRadioButtonHashMap(
        "Specification extraction", "ComputeSpecificationStrategy", false, false);
    final JRadioButtonHashMap bVBTStrategy = new JRadioButtonHashMap(
	"VBT Strategy", "VBTStrategy", false, false);

    ButtonGroup stratGroup = new ButtonGroup();
    ButtonGroup loopGroup = new ButtonGroup();
    ButtonGroup methodGroup = new ButtonGroup();
    ButtonGroup queryGroup = new ButtonGroup();
    ButtonGroup nonLinArithGroup = new ButtonGroup();
    JRadioButtonHashMap rdBut9;
    JRadioButtonHashMap rdBut10;
    JRadioButtonHashMap rdBut11;
    JRadioButtonHashMap rdBut12;
    JRadioButtonHashMap rdBut13;
    JRadioButtonHashMap rdBut14;
    private JRadioButtonHashMap queryNone;
    private JRadioButtonHashMap queryExpand;
    private JRadioButtonHashMap queryProgramsToRight;
    private JRadioButtonHashMap nonLinArithNone;
    private JRadioButtonHashMap nonLinArithCompletion;
    
    private KeYMediator mediator;
    

    JPanel javaDLOptionsPanel = new JPanel() {
        
        public void setEnabled(boolean enabled) {
             super.setEnabled(enabled);
             setChildrenEnabled(this, enabled);
        }

        private void setChildrenEnabled(Container container, boolean enabled) {
             for (int i=0; i<container.getComponentCount(); i++) {
                 Component comp = container.getComponent(i);
                 comp.setEnabled(enabled);
                 if (comp instanceof Container)
                     setChildrenEnabled((Container) comp, enabled);
             }
        }
    };
    
    Border loweredetched = 
        BorderFactory.createEtchedBorder(EtchedBorder.LOWERED);
  
    
    
    public StrategySelectionView () {        
        layoutPane();       
        refresh(mediator == null ? null : mediator.getSelectedProof());
	setVisible( true );
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentShown(java.awt.event.ComponentEvent e) {
                numLabel.refresh();
                maxSlider.refresh();
            }
        });
    }

    /** Build everything */    
    private void layoutPane() {        
        final JavaDLOptionListener javaDLOptionListener = new JavaDLOptionListener();
        bSimpleJavaCardDLStrategy.addPropertyChangeListener("enabled", 
                javaDLOptionListener);
        bSimpleJavaCardDLStrategy.addItemListener(javaDLOptionListener);        
        javaDLOptionsPanel.setEnabled(false);
        
        StratListener stratListener = new StratListener();
        OptListener optListener = new OptListener();
        
        GridBagLayout gbpanel0 = new GridBagLayout();
        GridBagConstraints gbcpanel0 = new GridBagConstraints();
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        javaDLOptionsPanel.setBorder(BorderFactory.createTitledBorder(""));
        GridBagLayout gbpanel5 = new GridBagLayout();
        GridBagConstraints gbcpanel5 = new GridBagConstraints();
        javaDLOptionsPanel.setLayout(gbpanel5);
        javaDLOptionsPanel.setLayout(new BoxLayout(javaDLOptionsPanel,
                                                   BoxLayout.Y_AXIS));
        
        
        JPanel loopPanel = new JPanel();
        loopPanel.setBorder(BorderFactory.createTitledBorder(
            loweredetched, "Loop treatment"));
        javaDLOptionsPanel.add(loopPanel);
        
        JPanel methodPanel = new JPanel();
        methodPanel.setBorder(BorderFactory.createTitledBorder(
            loweredetched, "Method treatment"));             
        javaDLOptionsPanel.add(methodPanel);

        rdBut9 = new JRadioButtonHashMap("Expand", StrategyProperties.LOOP_EXPAND, true, false);
        loopGroup.add(rdBut9);
        gbcpanel5.gridx = 0;
        gbcpanel5.gridy = 2;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut9, gbcpanel5);
        loopPanel.add(rdBut9);

        rdBut10 = new JRadioButtonHashMap("Invariant", StrategyProperties.LOOP_INVARIANT, false, false);
        loopGroup.add(rdBut10);
        gbcpanel5.gridx = 1;
        gbcpanel5.gridy = 2;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut10, gbcpanel5);
        loopPanel.add(rdBut10);

        
        rdBut11 = new JRadioButtonHashMap("None", StrategyProperties.LOOP_NONE, false, false);
        loopGroup.add(rdBut11);
        gbcpanel5.gridx = 2;
        gbcpanel5.gridy = 2;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut11, gbcpanel5);
        loopPanel.add(rdBut11);

        rdBut12 = new JRadioButtonHashMap("Expand", StrategyProperties.METHOD_EXPAND, true, false);
        methodGroup.add(rdBut12);
        gbcpanel5.gridx = 0;
        gbcpanel5.gridy = 6;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut12, gbcpanel5);
        methodPanel.add(rdBut12);

        rdBut13 = new JRadioButtonHashMap(
                "Contracts", StrategyProperties.METHOD_CONTRACT, false, false);
        methodGroup.add(rdBut13);
        gbcpanel5.gridx = 1;
        gbcpanel5.gridy = 6;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut13, gbcpanel5);
        methodPanel.add(rdBut13);

        rdBut14 = new JRadioButtonHashMap("None",
                StrategyProperties.METHOD_NONE, false, false);
        methodGroup.add(rdBut14);
        gbcpanel5.gridx = 2;
        gbcpanel5.gridy = 6;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(rdBut14, gbcpanel5);
        methodPanel.add(rdBut14);
        

        JPanel queryPanel = new JPanel();
        queryPanel.setBorder(BorderFactory.createTitledBorder(
            loweredetched, "Query treatment"));
        javaDLOptionsPanel.add(queryPanel);        
        
        queryExpand = new JRadioButtonHashMap("Expand", 
                StrategyProperties.QUERY_EXPAND, false, false);
        queryGroup.add(queryExpand);
        gbcpanel5.gridx = 0;
        gbcpanel5.gridy = 7;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(queryExpand, gbcpanel5);
        queryPanel.add(queryExpand);
        
        queryProgramsToRight = new JRadioButtonHashMap("Prog2Succ", 
                StrategyProperties.QUERY_PROGRAMS_TO_RIGHT, false, false);
        queryProgramsToRight.setToolTipText ( "<html>Move all program blocks to the" +
                                              " succedent.<br>" +
                                              " This is necessary" +
                                              " when query invocations<br>" +
                                              " are" +
                                              " supposed to be eliminated" +
                                              " using<br>" +
                                              " method contracts.</html>" );
        queryGroup.add(queryProgramsToRight);
        gbcpanel5.gridx = 0;
        gbcpanel5.gridy = 7;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(queryProgramsToRight, gbcpanel5);
        queryPanel.add(queryProgramsToRight);
                                      
        queryNone = new JRadioButtonHashMap("None", 
                StrategyProperties.QUERY_NONE, true, false);
        queryGroup.add(queryNone);
        gbcpanel5.gridx = 1;
        gbcpanel5.gridy = 7;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(queryNone, gbcpanel5);
        queryPanel.add(queryNone);            
        
        
        JPanel nonLinArithPanel = new JPanel();
        nonLinArithPanel.setBorder(BorderFactory.createTitledBorder(
            loweredetched, "Arithmetic treatment"));
        javaDLOptionsPanel.add(nonLinArithPanel);        
        
        nonLinArithNone = new JRadioButtonHashMap("Basic", 
                StrategyProperties.NON_LIN_ARITH_NONE, true, false);
        nonLinArithNone.setToolTipText("<html>" +
                "Basic arithmetic support:" +
                "<ul>" +
                "<li>Simplification of polynomial expressions</li>" +
                "<li>Computation of Gr&ouml;bner Bases for polynomials in the antecedent</li>" +
                "<li>(Partial) Omega procedure for handling linear inequations</li>" +
                "</ul>" +
                "</html>");
        nonLinArithGroup.add(nonLinArithNone);
        gbcpanel5.gridx = 1;
        gbcpanel5.gridy = 7;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(nonLinArithNone, gbcpanel5);
        nonLinArithPanel.add(nonLinArithNone);            
                                          
        nonLinArithCompletion = new JRadioButtonHashMap("Model Search", 
                StrategyProperties.NON_LIN_ARITH_COMPLETION, false, false);
        nonLinArithCompletion.setToolTipText ( "<html>" +
                "Support for non-linear inequations and model search.<br>" +
                "In addition, this performs:" +
                "<ul>" +
                "<li>Multiplication of inequations with each other</li>" +
                "<li>Systematic case distinctions (cuts)</li>" +
                "</ul>" +
                "This method is guaranteed to find counterexamples for<br>" +
                "invalid goals that only contain polynomial (in)equations.<br>" +
                "Such counterexamples turn up as trivially unprovable goals.<br>" +
                "It is also able to prove many more valid goals involving<br>" +
                "(in)equations, but will in general not terminate on such goals." +
                "</html>" );
        nonLinArithGroup.add(nonLinArithCompletion);
        gbcpanel5.gridx = 0;
        gbcpanel5.gridy = 7;
        gbcpanel5.gridwidth = 1;
        gbcpanel5.gridheight = 1;
        gbcpanel5.fill = GridBagConstraints.BOTH;
        gbcpanel5.weightx = 1;
        gbcpanel5.weighty = 0;
        gbcpanel5.anchor = GridBagConstraints.NORTH;
        gbpanel5.setConstraints(nonLinArithCompletion, gbcpanel5);
        nonLinArithPanel.add(nonLinArithCompletion);
        

        gbcpanel0.gridx = 4;
        gbcpanel0.gridy = 4;
        gbcpanel0.gridwidth = 3;
        gbcpanel0.gridheight = 7;
        gbcpanel0.fill = GridBagConstraints.BOTH;
        gbcpanel0.weightx = 1;
        gbcpanel0.weighty = 0;
        gbcpanel0.anchor = GridBagConstraints.NORTH;
        gbpanel0.setConstraints(javaDLOptionsPanel, gbcpanel0);

        Box box = Box.createVerticalBox();
        stratGroup.add(bSimpleFOLStrategy);
        stratGroup.add(bSimpleJavaCardDLStrategy);
        stratGroup.add(bSimplificationOfOCLStrategy);
        stratGroup.add(bVBTStrategy);
        stratGroup.add(bComputeSpecificationStrategy);
        javaDLOptionsPanel.setEnabled(bSimpleJavaCardDLStrategy.isSelected());
        bSimpleJavaCardDLStrategy.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                javaDLOptionsPanel.setEnabled(bSimpleJavaCardDLStrategy.isSelected());
            }
        });
        
        bSimpleFOLStrategy.setAlignmentX(Component.LEFT_ALIGNMENT);
        bSimpleJavaCardDLStrategy.setAlignmentX(Component.LEFT_ALIGNMENT);
        javaDLOptionsPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        bSimplificationOfOCLStrategy.setAlignmentX(Component.LEFT_ALIGNMENT);
        bComputeSpecificationStrategy.setAlignmentX(Component.LEFT_ALIGNMENT);
        
        JButton go = new JButton(Main.autoModeAction);

	JCheckBox resumeAutoMode = createResumeAutoModeCheckBox();
	
        Box b1 = new Box(BoxLayout.X_AXIS);
	b1.add(go);
	b1.add(Box.createHorizontalGlue());
	b1.add(resumeAutoMode);
        b1.setAlignmentX(Component.LEFT_ALIGNMENT);
                               
	box.add(b1);       
        box.add(numLabel);
        box.add(maxSlider);
        box.add(bSimpleFOLStrategy);
        box.add(bSimpleJavaCardDLStrategy);
        box.add(javaDLOptionsPanel);       
        box.add(bVBTStrategy);
	
        box.add(bSimplificationOfOCLStrategy);
        box.add(bComputeSpecificationStrategy);        
       
        // HACK but I really do not know enough about the GridBagLayout :-(
        final JPanel verticalGlue = new JPanel();
        verticalGlue.setPreferredSize(new Dimension(1, 1024));
        box.add(verticalGlue);

        this.add(box);
        //  add the ActionListener to the Buttons and ActionCommands 
        //  for identifying Buttons
        bSimpleFOLStrategy.addActionListener(stratListener);
        bSimpleJavaCardDLStrategy.addActionListener(stratListener);
        bSimplificationOfOCLStrategy.addActionListener(stratListener);
        bVBTStrategy.addActionListener(stratListener);
        bComputeSpecificationStrategy.addActionListener(stratListener);
        rdBut9.addActionListener(optListener);	
        rdBut10.addActionListener(optListener);       
        rdBut11.addActionListener(optListener);    
        rdBut12.addActionListener(optListener);       
        rdBut13.addActionListener(optListener);     
        rdBut14.addActionListener(optListener);
        queryExpand.addActionListener(optListener);
        queryProgramsToRight.addActionListener(optListener);
        queryNone.addActionListener(optListener);
        nonLinArithNone.addActionListener(optListener);
        nonLinArithCompletion.addActionListener(optListener);
    }
    
    
    
    private JCheckBox createResumeAutoModeCheckBox() {
	JCheckBox resumeAutoMode = new JCheckBox("Autoresume strategy");		
	resumeAutoMode.setToolTipText("Restart strategy after an interactive proof step?");
	resumeAutoMode.setBorderPaintedFlat(true);

/* Sorry, but the icon was concealing the checkbox nature of  this
thing. People were thinking it was a button.
	resumeAutoMode.setIcon
	    (IconFactory.resumeDisabledLogo(toolbarIconSize));		
*/
	
	resumeAutoMode.setMaximumSize(resumeAutoMode.getPreferredSize());
	resumeAutoMode.addItemListener(new ItemListener() {
		public void itemStateChanged(ItemEvent e) {
		    if (e.getStateChange() == ItemEvent.SELECTED) {
//			resumeAutoMode.
//			    setIcon(IconFactory.resumeLogo(toolbarIconSize));
			mediator.setResumeAutoMode(true);
		    } 
		    else if (e.getStateChange() == ItemEvent.DESELECTED) {
//			resumeAutoMode.
//			    setIcon(IconFactory.
//				    resumeDisabledLogo(toolbarIconSize));
			mediator.setResumeAutoMode(false);
		    }
		    else System.err.println(
		        "Automode checkbox undefined state: "+
			    e.getStateChange());
		    
	       }
	    });
	return resumeAutoMode;
    }
    
    

    public void setMediator(KeYMediator mediator) {
        this.mediator = mediator;
        mediator.addKeYSelectionListener(new SelectionChangedListener());
                
        final StrategyFactory defaultStrategyFactory = 
            mediator.getProfile().getDefaultStrategyFactory();
        
        getStrategyButton(defaultStrategyFactory.name().toString()).
            setSelected(true);
        
    }


    private final MaxAppLabel numLabel = new MaxAppLabel();
    private final MaxAppSlider maxSlider = new MaxAppSlider();
    
    
    private final class JavaDLOptionListener implements PropertyChangeListener, 
    ItemListener {
        public void propertyChange(PropertyChangeEvent evt) {
            if (evt.getNewValue() instanceof Boolean) {
                javaDLOptionsPanel.setEnabled
                (bSimpleJavaCardDLStrategy.isSelected() && 
                        ((Boolean)evt.getNewValue()).booleanValue());
            } else {
                javaDLOptionsPanel.
                    setEnabled(bSimpleJavaCardDLStrategy.isSelected() && 
                            Boolean.getBoolean(""+evt.getNewValue()));
            }
        }
        public void itemStateChanged(ItemEvent e) {
            if (e.getItem() == bSimpleJavaCardDLStrategy) {
                javaDLOptionsPanel.setEnabled(bSimpleJavaCardDLStrategy.isSelected() &&
                        bSimpleJavaCardDLStrategy.isEnabled()); 
            }
        }
    }

    class MaxAppLabel extends JLabel {
        private final static String TEXT = "Max. rule applications: ";
    
        public MaxAppLabel() {
            super(TEXT);
            setFont(getFont().deriveFont(9F));
            setAlignmentX(Component.LEFT_ALIGNMENT);
        }
    
        public void setLimit(int max) {
            setText(TEXT+max);
        }
        
        public void refresh() {
            setLimit(mediator.getMaxAutomaticSteps());
        }
    }
    
    class MaxAppSlider extends JSlider {
        private static final int MAX_RULE_APPS_LOG10 = 5;
        
        public MaxAppSlider() {
            super(JSlider.HORIZONTAL, 0, MAX_RULE_APPS_LOG10*9, 0);

            // set up slider labels
            Hashtable labelTable = new Hashtable();

            for ( int n = 0; n <= MAX_RULE_APPS_LOG10; n++ ) {
                int val = (int)Math.pow(10, n);
                JLabel l = new JLabel ( ""+val );
                l.setFont(l.getFont().deriveFont(9F));
                labelTable.put(new Integer ( n*9 ), l);
            }

            setLabelTable ( labelTable );
            setPaintLabels ( true );

            // show ticks
            setMajorTickSpacing ( 9 );
            setMinorTickSpacing ( 1 );
            setPaintTicks ( true );

            // add change listener
            addChangeListener( new ChangeListener() {
                public void stateChanged ( ChangeEvent e ) {
                    int n = getValue();
                    int major = n / 9;
                    int minor = n % 9;
                    int val = (minor+1)*(int)Math.pow(10, major);
                    numLabel.setLimit( val );
                    mediator.setMaxAutomaticSteps ( val );
                }
            });

            setAlignmentX(Component.LEFT_ALIGNMENT);
        }
        
        public void setPos(int maxRuleApps) {
            if ( maxRuleApps > 0 ) {
                int major = (int)(Math.log ( maxRuleApps ) / Math.log ( 10 ));
                int minor = maxRuleApps / (int)Math.pow(10, major) - 1;
                int initPos = Math.max ( 0, Math.min ( MAX_RULE_APPS_LOG10*9,
                                                       major * 9 + minor ));
                setValue(initPos);
            }
        }
        
        public void refresh() {
            setPos(mediator.getMaxAutomaticSteps());
        }
    
    }

    /**
     * TODO: here should be a settings listener listening to strategy setting changes
     * Therefore we have to wait until ProofSettings have been refactored (see KeY-Wiki) 
     * 
     *(Currently changes made in the old slider are not reported to this view)
     */
    private final class SelectionChangedListener 
        implements KeYSelectionListener {
        
        public void selectedNodeChanged(KeYSelectionEvent e) {}

        public void selectedProofChanged(KeYSelectionEvent e) {
            refresh(e.getSource().getSelectedProof());            
        }        
    }

    
    /** performs a refresh of all elements in this tab */
    public void refresh(Proof proof) {
        if (proof == null) {            
            enableAll(false);                
        } else {
            maxSlider.refresh();
            numLabel.refresh();
            enableAll(true);
            String activeS = proof.getActiveStrategy().name().toString();
            JRadioButton bactive = JRadioButtonHashMap.getButton(activeS);
            bactive.setSelected(true);
            
            StrategyProperties p = proof.
                getSettings().getStrategySettings().
                    getActiveStrategyProperties();
            
            String activeLoopOptions = p.getProperty(StrategyProperties.LOOP_OPTIONS_KEY);
            JRadioButton bLoopActive = JRadioButtonHashMap.getButton(
                    activeLoopOptions);
            bLoopActive.setSelected(true);
            String activeMethodOptions = p.getProperty(StrategyProperties.METHOD_OPTIONS_KEY);
            JRadioButton bMethodActive = JRadioButtonHashMap.getButton(
                activeMethodOptions);
            bMethodActive.setSelected(true);   
            String activeQueryOptions = p.getProperty(StrategyProperties.QUERY_OPTIONS_KEY);
            JRadioButton bQueryActive = JRadioButtonHashMap.getButton(
                activeQueryOptions);
            bQueryActive.setSelected(true);   
            String activeNonLinArithOptions = p.getProperty(StrategyProperties.NON_LIN_ARITH_OPTIONS_KEY);
            JRadioButton bNonLinArithActive = JRadioButtonHashMap.getButton(
                activeNonLinArithOptions);
            bNonLinArithActive.setSelected(true);   
        }
    }

    private JRadioButton getStrategyButton(String name) {
        return JRadioButtonHashMap.getButton(name);      
    }
    
    /**
     * enables or disables all components
     * @param enable boolean saying whether to activate or
     * deactivate the components
     */
    private void enableAll(boolean enable) {             
        maxSlider.setEnabled(enable);                        
        if (mediator != null) {                   
            final IteratorOfStrategyFactory supportedStrategies = 
               mediator.getProfile().supportedStrategies().iterator();
            while (supportedStrategies.hasNext()) {                  
                final StrategyFactory next = supportedStrategies.next();              
                getStrategyButton(next.name().toString()).setEnabled(enable);               
            }
        }
    }

    public Strategy getStrategy(String strategyName, Proof proof,
            StrategyProperties properties) {
        if (mediator != null) {        
            final IteratorOfStrategyFactory supportedStrategies = 
               mediator.getProfile().supportedStrategies().iterator();
            while (supportedStrategies.hasNext()) {                
                final StrategyFactory s = supportedStrategies.next();
                if (strategyName.equals(s.name().toString())) {                    
                    return s.create(proof, properties);
                }
            }
        }
        return null;
    }
  
    /**
     * @return
     */
    private StrategyProperties getProperties() {
        StrategyProperties p = new StrategyProperties();        
        p.setProperty( StrategyProperties.LOOP_OPTIONS_KEY, 
                       loopGroup.getSelection().getActionCommand());
        p.setProperty( StrategyProperties.METHOD_OPTIONS_KEY, 
                       methodGroup.getSelection().getActionCommand());
        p.setProperty( StrategyProperties.QUERY_OPTIONS_KEY, 
                       queryGroup.getSelection().getActionCommand());
        p.setProperty( StrategyProperties.NON_LIN_ARITH_OPTIONS_KEY, 
                       nonLinArithGroup.getSelection().getActionCommand());
        
        return p;
    }

    /**
     * 
     */
    private void updateStrategySettings(String strategyName) {
        final Proof proof = mediator.getProof();
        final StrategyProperties p = getProperties();
        final Strategy strategy = getStrategy(strategyName, proof, p);

        ProofSettings.DEFAULT_SETTINGS.getStrategySettings().
        setStrategy(strategy.name());
        ProofSettings.DEFAULT_SETTINGS.getStrategySettings().
            setActiveStrategyProperties(p);
        
        proof.getSettings().getStrategySettings().
        setStrategy(strategy.name());
        proof.getSettings().getStrategySettings().
            setActiveStrategyProperties(p);
        
        proof.setActiveStrategy(strategy);
    }

    public class StratListener implements ActionListener { 
        public void actionPerformed(ActionEvent e) {            
            updateStrategySettings(e.getActionCommand());
        }
    }

    public class OptListener implements ActionListener { 
        public void actionPerformed(ActionEvent e) {   
            updateStrategySettings(mediator.getProof().
                    getActiveStrategy().name().toString());
        }
    }

}
