// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.op.Location;
import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.logic.op.SVSubstitute;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.rule.MatchConditions;
import de.uka.ilkd.key.rule.Taclet;
import de.uka.ilkd.key.util.Debug;

public class BasicLocationDescriptor implements LocationDescriptor {
    
    private final Term fma;
    private final Term locTerm;
    private final static Term trueTerm = TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);
    
    public BasicLocationDescriptor(Term fma, Term locTerm) {
        assert fma != null && fma.sort() == Sort.FORMULA && locTerm != null;
        if (!(locTerm.op() instanceof Location)) {
            throw new IllegalArgumentException("Expected a location, but " + locTerm + 
                    " is a " + locTerm.op().getClass().getName());
        }
        this.fma = fma;
        this.locTerm = locTerm;
    }
    
    public BasicLocationDescriptor(Term locTerm) {
        this(trueTerm, locTerm);
    }
    
    public Term getFormula() {
        return fma;
    }
    
    public Term getLocTerm() {
        return locTerm;
    }    
    
    public boolean equals(Object o) {
        if(!(o instanceof BasicLocationDescriptor)) {
            return false;
        }       
        BasicLocationDescriptor ld = (BasicLocationDescriptor) o;
        return fma.equals(ld.fma) && locTerm.equals(ld.locTerm);
    }
    
    public int hashCode() {
        return fma.hashCode() + locTerm.hashCode();
    }
    
    public String toString() {
        return (fma.equals(trueTerm) 
                ? locTerm.toString() 
                : "(" + fma + "," + locTerm + ")");
    }

    public MatchConditions match(SVSubstitute descriptor, 
            MatchConditions mc, Services services) {
        if (descriptor instanceof BasicLocationDescriptor) {
            final BasicLocationDescriptor locDescr = (BasicLocationDescriptor)descriptor;
            
            final MatchConditions formulaMC = Taclet.matchHelp(locDescr.getFormula(), 
                    getFormula(), false, mc, null, services, Constraint.BOTTOM); 
            //Bug: Fix user constraint
            
            if (formulaMC == null) {
                Debug.out("Match FAILED. Guards do not match ", this, descriptor);
                return null;
            }
            
            return Taclet.matchHelp(locDescr.getLocTerm(), 
                    getLocTerm(), false, formulaMC, null, services, Constraint.BOTTOM);
        }
        Debug.out("Match FAILED. Incompatible types (template, source)", this, descriptor);
        return null;
    }
}
