// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.logic.op.ListOfQuantifiableVariable;
import de.uka.ilkd.key.logic.op.SLListOfQuantifiableVariable;
/** visitor for <t> execPostOrder </t> of 
 * {@link de.uka.ilkd.key.logic.Term}. Called with that method
 * on a term, the visitor delivers all bound variables starting with
 * the variables bound to a given subterm and its successive subterms.
 */
public class BoundVarsVisitor extends Visitor{
  
    private ListOfQuantifiableVariable bdVars =
	SLListOfQuantifiableVariable.EMPTY_LIST;  

    // if forSubTerm==-1, then the bound vars for all subterms are
    // taken, otherwise the vars bound for the subterm with that
    // number are taken. 
    private int forSubTerm=-1;

    /**
     * creates a visitor starting to collect the bound variables for
     * the subterm determined by the given number and the successive
     * subterms. If forSubTerm equals -1 then the bound variables for all
     * subterms are considered.
     */
    public BoundVarsVisitor(int forSubTerm) {
	this.forSubTerm=forSubTerm;
    }

    /**
     * creates a Visitor that collects all bound variables for the subterms
     * of the term it is called from.
     */
    public BoundVarsVisitor() {
	forSubTerm=-1;
    }

    private void addVarsBoundHere(Term visited, int subTerm) {
	for (int j=0; j<visited.varsBoundHere(subTerm).size(); j++) {
	    bdVars=bdVars.prepend(visited.varsBoundHere(subTerm)
				  .getQuantifiableVariable(j));	    
	}
    }

    /**
     * only called by execPostOrder in Term.
     */
    public void visit(Term visited) {
	if (forSubTerm!=-1) {	    
	    addVarsBoundHere(visited, forSubTerm);
	    forSubTerm=-1;
	} else {
	    for (int i=0; i<visited.arity(); i++) {
		addVarsBoundHere(visited, i);
	    }
	}
    }
    
    /**
     * returns all the bound variables that have been stored
     */
    public ListOfQuantifiableVariable getBoundVariables(){
	return bdVars;
    }

}
