// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//
/*
 * Created on 22.12.2004
 */
package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.AbstractMetaOperator;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.op.ListSV;
import de.uka.ilkd.key.logic.op.SVSubstitute;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.rule.IteratorOfObject;
import de.uka.ilkd.key.rule.ListOfObject;
import de.uka.ilkd.key.rule.MatchConditions;
import de.uka.ilkd.key.rule.UpdateSimplifier;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

/**
 * Creates an anonymising update for a modifies clause.
 */
public class IntroNewAnonUpdateOp extends AbstractMetaOperator {
    
    public IntroNewAnonUpdateOp() {
        super(new Name("#introNewAnonUpdate"), 3);
    }

    
    /* (non-Javadoc)
     * @see de.uka.ilkd.key.logic.op.MetaOperator#calculate(de.uka.ilkd.key.logic.Term, de.uka.ilkd.key.rule.inst.SVInstantiations, de.uka.ilkd.key.java.Services)
     */
    public Term calculate(Term term, SVInstantiations svInst, Services services) {
        ListSV modifiesSV = (ListSV) term.sub(0).op();
        ListOfObject locationList
                = (ListOfObject) svInst.getInstantiation(modifiesSV);
        LocationDescriptor[] locations
                = new LocationDescriptor[locationList.size()];
        IteratorOfObject it = locationList.iterator();
        for(int i = 0; i < locations.length; i++) {
            locations[i] = (LocationDescriptor) it.next();
        }
        
        ListSV anon = (ListSV) term.sub(2).op();
        ListOfObject uninFunctionList 
                = (ListOfObject) svInst.getInstantiation(anon);
        Function[] uninFunctions
                = new Function[uninFunctionList.size()];
        it = uninFunctionList.iterator();
        for(int i = 0; i < uninFunctions.length; i++) {
            uninFunctions[i] = (Function) it.next();
        }
        
        UpdateFactory uf = new UpdateFactory(services, new UpdateSimplifier());
        AnonymisingUpdateFactory auf = new AnonymisingUpdateFactory(uf);
        
        return auf.createAnonymisingUpdateTerm(locations, 
                                               uninFunctions, 
                                               term.sub(1),
				               services);
    }
                
    /* (non-Javadoc)
     * @see de.uka.ilkd.key.logic.op.Operator#validTopLevel(de.uka.ilkd.key.logic.Term)
     */
    public boolean validTopLevel(Term term) {
        return term.arity()==3 && term.sub(1).sort()==Sort.FORMULA;
    }

    
    /* (non-Javadoc)
     * @see de.uka.ilkd.key.logic.op.Operator#sort(de.uka.ilkd.key.logic.Term[])
     */
    public Sort sort(Term[] term) {
        return term[1].sort();
    }
    

    /* (non-Javadoc)
     * @see de.uka.ilkd.key.logic.op.Operator#isRigid(de.uka.ilkd.key.logic.Term)
     */
    public boolean isRigid(Term term) {
        return false;
    }

    
    /** (non-Javadoc)
     * by default meta operators do not match anything 
     * @see de.uka.ilkd.key.logic.op.Operator#match(SVSubstitute, de.uka.ilkd.key.rule.MatchConditions, de.uka.ilkd.key.java.Services)
     */
    public MatchConditions match(SVSubstitute subst, MatchConditions mc,
            Services services) {
        return null;
    }
}
