// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
//
//
/**
 * tests the symbolic execution of the program meta constructs
 */
package de.uka.ilkd.key.rule.metaconstruct;

import junit.framework.TestCase;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.expression.Assignment;
import de.uka.ilkd.key.java.expression.literal.BooleanLiteral;
import de.uka.ilkd.key.java.reference.TypeRef;
import de.uka.ilkd.key.java.statement.Break;
import de.uka.ilkd.key.java.statement.LabeledStatement;
import de.uka.ilkd.key.java.statement.LoopStatement;
import de.uka.ilkd.key.java.visitor.JavaASTCollector;
import de.uka.ilkd.key.logic.NamespaceSet;
import de.uka.ilkd.key.logic.ProgramElementName;
import de.uka.ilkd.key.rule.TacletForTests;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

public class TestProgramMetaConstructs extends TestCase {

    private Recoder2KeY r2k;

    public TestProgramMetaConstructs(String name) {
	super(name);
	de.uka.ilkd.key.util.Debug.ENABLE_DEBUG = false;
    }

    public void setUp() {
	r2k = new Recoder2KeY(new Services(),
			      new NamespaceSet());
    }

        
    public void testDoBreak() {
	LabeledStatement labeledBlock = (LabeledStatement)
	    ((StatementBlock)TacletForTests.parsePrg
	 //("{l1:l2:{l3:{l4:break l3; int i = 0;} int j=1;}}")).getChildAt(0);
	     ("{l4:break l3; int i = 0; int j=1;}")).getChildAt(0);
	DoBreak rmLabel = 
	    new DoBreak(labeledBlock);
	
	ProgramElement result = rmLabel.
	    symbolicExecution
	    (rmLabel.body(), new Services(),
	     SVInstantiations.EMPTY_SVINSTANTIATIONS);
	assertTrue(result instanceof Break);
    }


    /** tests  AST walkers */
    public void xtestASTWalker() {
 	ProgramElement block = 
 	    TacletForTests.parsePrg("{int a=5; test1:test2:while (true) "+
 			  "{test3: {int j=3;}}}");
 	JavaASTCollector coll = new JavaASTCollector
 	    (block, 
 	     de.uka.ilkd.key.java.statement.LabeledStatement.class);
 	coll.start();
 	assertTrue(coll.getNodes().size() == 3);

 	ProgramElement block2 = 
 	    TacletForTests.parsePrg("{while(true) {if (true) break; else continue;}}");
 	WhileLoopTransformation trans = 
 	    new WhileLoopTransformation(block2, 
					new ProgramElementName("l1"), 
					new ProgramElementName("l2"));
 	trans.start();
 	System.out.println("Result:"+trans);	
    }

    public void testTypeOf() { //this is no really sufficient test
	Services services = new Services();
	//but I can't access programs here
 	StatementBlock block = (StatementBlock)
 	    TacletForTests.parsePrg(" { int i; int j; i=j; }");
	Expression expr=(Expression)((Assignment)block.getStatementAt(2))
	    .getChildAt(1);
	ProgramMetaConstruct typeof = new TypeOf(expr);
	assertTrue(((TypeRef)typeof.symbolicExecution(expr, services, 
		 SVInstantiations.EMPTY_SVINSTANTIATIONS)).getName()
	       .toString().equals("int"));
    }

    public void testBugId183 () {
	StatementBlock bl = ( StatementBlock ) TacletForTests.parsePrg ( "{ while ( true ) {} }" );
	LoopStatement l = (LoopStatement) bl.getChildAt (0);
	UnwindLoop wlt = new UnwindLoop ( l );

	SVInstantiations inst = SVInstantiations.EMPTY_SVINSTANTIATIONS;
	try {
	    wlt.symbolicExecution( l, new Services (), inst );
	} catch ( java.util.NoSuchElementException e ) {
	    assertTrue ( " Problem with empty while-blocks. See Bug #183 ", false); 
	}
	
    }

}
