// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.strategy.feature;

import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.rule.RuleApp;
import de.uka.ilkd.key.strategy.LongRuleAppCost;
import de.uka.ilkd.key.strategy.RuleAppCost;
import java.util.HashMap;

/**
 * A feature that computes the depth of the find-position of a taclet (top-level
 * positions have depth zero) 
 * 
 * TODO: eliminate this class and use term features instead
 */
public class DepthGreaterThanFeature extends BinaryFeature {

    private static HashMap features = new HashMap(10);

    private int greaterThan = 0;

    public static Feature create(int greaterThan) {
	final Integer gt = new Integer(greaterThan);
	Feature f = (Feature) features.get(gt);
	if (f == null) {
	    f = new DepthGreaterThanFeature(greaterThan);
	    features.put(gt, f);
	} 
	return f;
    }

    private DepthGreaterThanFeature (int greaterThan) {
	this.greaterThan = greaterThan;
    }
    
    /**
     * Compute whether the result of the feature is zero (<code>true</code>)
     * or infinity (<code>false</code>)
     * 
     * @param app
     *            the RuleApp
     * @param pos
     *            position where <code>app</code> is to be applied
     * @param goal
     *            the goal on which <code>app</code> is to be applied
     * @return true iff the the result of the feature is supposed to be zero.
     */
    protected boolean filter ( RuleApp app,
			       PosInOccurrence pos,
			       Goal goal ) {
        assert pos != null : "Feature is only applicable to rules with find";

        return pos.depth() > greaterThan;    
    }

}
